# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.
"""Communities user moderation."""

from .actions import on_block, on_restore

__all__ = ("on_block", "on_restore")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022-2024 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community PID slug field."""
from uuid import UUID

from invenio_records.dictutils import filter_dict_keys
from invenio_records.systemfields import SystemField
from sqlalchemy.orm.exc import NoResultFound


def is_valid_uuid(value):
    """Check if the provided value is a valid UUID."""
    try:
        UUID(str(value))
        return True
    except (ValueError, AttributeError, TypeError):
        return False


class ParentCommunityField(SystemField):
    """System field for parent community."""

    def __init__(self, key="parent"):
        """Create a new ParentCommunityField instance."""
        super().__init__(key=key)

    def obj(self, instance):
        """Get the access object."""
        obj = self._get_cache(instance)
        if obj is not None:
            return obj

        value = self.get_dictkey(instance)
        if value:
            obj = instance.get_record(value["id"])
        else:
            obj = None

        self._set_cache(instance, obj)
        return obj

    def set_obj(self, record, obj):
        """Set the access object."""
        # Check if obj is None and remove 'parent' key from record
        if obj is None:
            record.pop("parent", None)
            self._set_cache(record, None)
            return

        if isinstance(obj, type(record)):
            parent_community = obj
        elif is_valid_uuid(obj):
            try:
                # Attempt to retrieve the community to confirm its existence
                parent_community = record.get_record(obj)
            except NoResultFound as e:
                raise ValueError("Community does not exist.") from e
        else:
            raise ValueError("Invalid parent community.")

        # Store the community ID in the record JSON
        record["parent"] = {"id": str(parent_community.id)}
        # Cache the community object
        self._set_cache(record, parent_community)

    def __get__(self, record, owner=None):
        """Get the record's access object."""
        if record is None:
            # access by class
            return self

        # access by object
        return self.obj(record)

    def __set__(self, record, obj):
        """Set the records access object."""
        self.set_obj(record, obj)

    def post_dump(self, record, data, dumper=None):
        """After dumping, dereference the parent community."""
        parent_community = getattr(record, self.attr_name)
        if parent_community:
            dump = parent_community.dumps()
            data[self.key] = filter_dict_keys(
                dump,
                keys=[
                    "uuid",
                    "created",
                    "updated",
                    "id",
                    "slug",
                    "theme",
                    "version_id",
                    "children.allow",
                    "metadata.title",
                    "metadata.type",
                    "metadata.website",
                    "metadata.organizations",
                    "metadata.funding",
                ],
            )

    def post_load(self, record, data, loader=None):
        """Laod the parent community using the OS data (preventing a DB query)."""
        if data.get("parent"):
            record.parent = record.loads(data["parent"])



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2024 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community theme template loader."""

from flask import current_app
from flask.templating import DispatchingJinjaLoader
from invenio_app.helpers import ThemeJinjaLoader
from jinja2 import ChoiceLoader
from werkzeug.utils import cached_property


class CommunityThemeJinjaLoader(ThemeJinjaLoader):
    """Community theme choice template loader.

    This loader acts as a wrapper for any type of Jinja loader. Before doing a
    template lookup, the loader sequentially applies prefixes to the template
    name, until a template source is found.

    The prefixes are defined via the ``APP_THEME`` configuration variable. This loader
    ensures that the `<brand>` theme takes precedence in the theme lookup.
    """

    def __init__(self, app, loader, brand):
        """Initialize loader.

        :param brand: community brand to be looked up first.
        """
        self.brand = brand
        super().__init__(app, loader)

    @cached_property
    def prefixes(self):
        """Return the active prefixes to be used for template lookup.

        Ensure `community.theme.brand` takes precedence in the list.
        """
        theme = self.app.config.get("APP_THEME", [])
        if isinstance(theme, str):
            # fe. semantic-ui/themes/horizon
            theme = f"themes/{self.brand}"
            theme = [theme]
        elif isinstance(theme, list):
            theme = [f"themes/{self.brand}" for item in theme]
        return theme


class CommunityThemeChoiceJinjaLoader(ChoiceLoader):
    """Community theme choice template loader.

    This loader acts as a wrapper for any type of Jinja ChoiceLoader. Dictates an order
    of theme loaders in the following order:
    - Community theme <brand>
    - Community theme default
    - Default `app.jinja_env.loader`
    """

    def __init__(self, brand):
        """Initialize loader.

        :param brand: community brand to be load .
        """
        self.app = current_app._get_current_object()

        community_theme_loader = CommunityThemeJinjaLoader(
            self.app, DispatchingJinjaLoader(self.app), brand
        )
        default_loader = CommunityThemeJinjaLoader(
            self.app, DispatchingJinjaLoader(self.app), "default"
        )
        super().__init__(
            loaders=[community_theme_loader, default_loader, self.app.jinja_env.loader]
        )



import pytest
from invenio_records_resources.services.errors import PermissionDeniedError


@pytest.fixture(scope="function")
def groups_disabled(app):
    app.config["COMMUNITIES_GROUPS_ENABLED"] = False
    yield
    app.config["COMMUNITIES_GROUPS_ENABLED"] = True


def test_invite_member_with_groups_disabled(
    member_service, community, owner, new_user, db, groups_disabled
):
    """Invite a user when the groups are disabled."""
    data = {
        "members": [{"type": "user", "id": str(new_user.id)}],
        "role": "reader",
    }
    member_service.invite(owner.identity, community._record.id, data)


@pytest.mark.parametrize(
    "actor,role",
    [
        ("owner", "owner"),
        ("owner", "manager"),
        ("owner", "curator"),
        ("owner", "reader"),
        ("manager", "manager"),
        ("manager", "curator"),
        ("manager", "reader"),
    ],
)
def test_add_member_with_groups_disabled(
    member_service, community, members, group, actor, role, db, groups_disabled
):
    """Test that the COMMUNITIES_GROUPS_ENABLED flag allows/forbids to add
    members.
    """
    data = {
        "members": [{"type": "group", "id": group.name}],
        "role": role,
    }

    assert pytest.raises(
        PermissionDeniedError,
        member_service.add,
        members[actor].identity,
        community._record.id,
        data,
    )
